'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

exports.default = init;

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

var _lodash = require('lodash.merge');

var _lodash2 = _interopRequireDefault(_lodash);

var _lodash3 = require('lodash.omit');

var _lodash4 = _interopRequireDefault(_lodash3);

var _verifier = require('./verifier');

var _verifier2 = _interopRequireDefault(_verifier);

var _passportLdapauth = require('passport-ldapauth');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var debug = (0, _debug2.default)('feathers-authentication-ldap');
var defaults = {
  name: 'ldap',
  server: {
    url: 'ldap://localhost:389',
    bindDn: 'cn=anonymous',
    bindCredentials: '',
    searchBase: 'dc=de',
    searchFilter: '(uid={{username}})',
    searchAttributes: null
  },
  passReqToCallback: true
};

var JWTVerifier = function () {
  function JWTVerifier() {
    _classCallCheck(this, JWTVerifier);
  }

  _createClass(JWTVerifier, [{
    key: 'verify',

    // JWT Verifier replacement
    // to disable population from user service
    value: function verify(req, payload, done) {
      done(null, { payload: payload });
    }
  }]);

  return JWTVerifier;
}();

// Export ldap-auth init function


function init() {
  var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  return function ldapAuth() {
    var app = this;
    var _super = app.setup;

    if (!app.passport) {
      throw new Error('Can not find app.passport. Did you initialize feathers-authentication before feathers-authentication-ldap?');
    }

    // Construct ldapSettings for passport ldap strategy
    var name = options.name || defaults.name;
    var authOptions = app.get('auth') || {};
    var ldapOptions = authOptions[name] || {};
    var ldapSettings = (0, _lodash2.default)({}, defaults, ldapOptions, (0, _lodash4.default)(options, ['Verifier']));
    var Verifier = options.Verifier || _verifier2.default;

    // plugin setup: register strategy in feathers passport
    app.setup = function () {
      // be sure feathers setup was called
      var result = _super.apply(this, arguments);
      var verifier = new Verifier(app, ldapSettings);

      if (!verifier.verify) {
        throw new Error('Your verifier must implement a \'verify\' function. It should have the same signature as function(request, user, done)');
      }

      // Register 'ldap' strategy with passport
      debug('Registering ldap authentication strategy with options:', ldapSettings);
      app.passport.use(ldapSettings.name, new _passportLdapauth.Strategy(ldapSettings, verifier.verify.bind(verifier)));
      app.passport.options(ldapSettings.name, ldapSettings); // do we need this ??

      return result;
    };
  };
}

// Exposed Modules
Object.assign(init, {
  defaults: defaults,
  Verifier: _verifier2.default,
  JWTVerifier: JWTVerifier
});
module.exports = exports['default'];